@@ -32,6 +32,7 @@ import grp
 import io
 import locale
 import logging
+import logging.handlers
 import re
 import os
 import select
@@ -41,6 +42,7 @@ import subprocess
 import sys
 
 
+from datetime import date
 from email.message import Message
 from gettext import gettext as _
 from io import StringIO
@@ -441,7 +443,7 @@ def upgrade_in_minimal_steps(cache, pkgs_to_upgrade, blacklist,
     to_upgrade = set(pkgs_to_upgrade)
     while True:
         # find smallest set
-        smallest_partition = to_upgrade
+        smallest_partition = set()
         for pkgname in to_upgrade:
             if SIGNAL_STOP_REQUEST:
                 logging.warning("SIGNAL received, stopping")
@@ -468,12 +470,18 @@ def upgrade_in_minimal_steps(cache, pkgs_to_upgrade, blacklist,
                 logging.debug("found leaf package %s" % pkg.name)
                 smallest_partition = changes
                 break
-            if len(changes) < len(smallest_partition):
+            if (len(changes) < len(smallest_partition) or
+                    len(smallest_partition) == 0):
                 logging.debug("found partition of size %s (%s)" % (
                     len(changes), changes))
                 smallest_partition = changes
             cache.clear()
 
+        if len(smallest_partition) == 0:
+            logging.info(
+                _("Some packages can not be upgraded: %s") % (to_upgrade))
+            break
+
         # write progress log information
         if len(pkgs_to_upgrade) > 0:
             percent = ((len(pkgs_to_upgrade) - len(to_upgrade)) /
@@ -487,6 +495,14 @@ def upgrade_in_minimal_steps(cache, pkgs_to_upgrade, blacklist,
         logging.debug("applying set %s" % smallest_partition)
         rewind_cache(cache, [cache[name] for name in smallest_partition])
 
+        # one last check, just to be sure
+        if not check_changes_for_sanity(cache, allowed_origins, blacklist,
+                                        whitelist):
+            logging.error("Failed sanity check while attempting to commit: "
+                          "this should never happen")
+            cache.clear()
+            return False
+
         res, error = cache_commit(cache, logfile_dpkg, verbose, install_log)
         if error:
             if verbose:
@@ -957,6 +973,25 @@ def _setup_logging(options):
     if apt_pkg.config.find("Unattended-Upgrade::Mail", ""):
         mem_log_handler = logging.StreamHandler(mem_log)
         logger.addHandler(mem_log_handler)
+    # Configure syslog if necessary
+    syslogEnable = apt_pkg.config.find_b("Unattended-Upgrade::SyslogEnable",
+                                         False)
+    if syslogEnable:
+        syslogFacility = apt_pkg.config.find(
+            "Unattended-Upgrade::SyslogFacility",
+            "daemon")
+        syslogHandler = logging.handlers.SysLogHandler(
+            address='/dev/log',
+            facility=syslogFacility)
+        syslogHandler.setFormatter(
+            logging.Formatter("unattended-upgrade: %(message)s"))
+        if syslogFacility.lower() in syslogHandler.facility_names.keys():
+            logger.addHandler(syslogHandler)
+            logging.info("Enabled logging to syslog via %s facility "
+                         % syslogFacility)
+        else:
+            logging.warning("Syslog facility %s was not found"
+                            % syslogFacility)
     return mem_log
 
 
@@ -1094,15 +1129,13 @@ def get_auto_removable(cache):
                 if pkg.is_auto_removable])
 
 
-def do_auto_remove(cache, auto_removable, logfile_dpkg,
-                   verbose=False, dry_run=False):
+def do_auto_remove(cache, auto_removable, logfile_dpkg, verbose=False):
     if not auto_removable:
         return True
 
     for pkgname in auto_removable:
-        if not dry_run:
-            logging.debug("marking %s for remove" % pkgname)
-            cache[pkgname].mark_delete()
+        logging.debug("marking %s for remove" % pkgname)
+        cache[pkgname].mark_delete()
     logging.info(_("Packages that are auto removed: '%s'"),
                  " ".join(sorted(auto_removable)))
 
@@ -1129,6 +1162,30 @@ def clean_downloaded_packages(fetcher):
                 pass
 
 
+def is_update_day():
+    # check if patch days are configured
+    patch_days = apt_pkg.config.value_list("Unattended-Upgrade::Update-Days")
+    if not patch_days:
+        return True
+    # validate patch days
+    today = date.today()
+    # abbreviated localized dayname
+    if today.strftime("%a") in patch_days:
+        return True
+    # full localized dayname
+    if today.strftime("%A") in patch_days:
+        return True
+    # by number (Sun: 0, Mon: 1, ...)
+    if today.strftime("%w") in patch_days:
+        return True
+    # today is not a patch day
+    logging.info(
+        "Skipping update check: today is '%s,%s,%s' but patch days are '%s'",
+        today.strftime("%w"), today.strftime("%a"), today.strftime("%A"),
+        patch_days)
+    return False
+
+
 def main(options, rootdir=""):
     # useful for testing
     if rootdir:
@@ -1137,6 +1194,10 @@ def main(options, rootdir=""):
     # setup logging
     mem_log = _setup_logging(options)
 
+    # check if today is a patch day
+    if not is_update_day():
+        return
+
     # format (origin, archive), e.g. ("Ubuntu","dapper-security")
     allowed_origins = get_allowed_origins()
 
@@ -1247,9 +1308,6 @@ def main(options, rootdir=""):
     res = fetcher.run()
     logging.debug("fetch.run() result: %s", res)
 
-    if options.download_only:
-        return
-
     if dpkg_conffile_prompt():
         # now check the downloaded debs for conffile conflicts and build
         # a blacklist
@@ -1338,9 +1396,7 @@ def main(options, rootdir=""):
         write_stamp_file()
         # check if we couldn't reboot on previous run because
         # a user was logged-in at this time
-        # never reboot during a dry run
-        if not options.dry_run:
-            reboot_if_requested_and_needed()
+        reboot_if_requested_and_needed()
         return
 
     # check if its configured for install on shutdown, if so, the
@@ -1397,8 +1453,7 @@ def main(options, rootdir=""):
         auto_removals = get_auto_removable(cache)
         pkg_install_success &= do_auto_remove(
             cache, auto_removals, logfile_dpkg,
-            options.verbose or options.debug,
-            options.dry_run)
+            options.verbose or options.debug)
     # the user wants *only new* auto-removals to be removed
     elif apt_pkg.config.find_b(
             "Unattended-Upgrade::Remove-New-Unused-Dependencies", True):
@@ -1407,8 +1462,7 @@ def main(options, rootdir=""):
         auto_removals = new_pending_autoremovals - previous_autoremovals
         pkg_install_success &= do_auto_remove(
             cache, auto_removals, logfile_dpkg,
-            options.verbose or options.debug,
-            options.dry_run)
+            options.verbose or options.debug)
 
     logging.debug("InstCount=%i DelCount=%i BrokenCount=%i"
                   % (cache._depcache.inst_count,
@@ -1460,9 +1514,6 @@ if __name__ == "__main__":
     parser.add_option("", "--dry-run",
                       action="store_true", default=False,
                       help=_("Simulation, download but do not install"))
-    parser.add_option("", "--download-only",
-                      action="store_true", default=False,
-                      help=_("Only download, do not even try to install."))
     parser.add_option("", "--minimal-upgrade-steps",
                       action="store_true", default=False,
                       help=_("Upgrade in minimal steps (and allow "
